/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package eb;

/**
 * Accountbean is an entity bean with "container managed persistence". 
 * The state of an instance is stored into a relational database. 
 * The following table should exist:
 * create table ACCOUNT (ACCNO    integer primary key,
 *                       CUSTOMER varchar(30),
 *                       BALANCE  number(15,4));
 */

import java.rmi.RemoteException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;


public class AccountImplBean implements EntityBean {

    // Keep the reference on the EntityContext
    protected EntityContext entityContext;

    // Object state
    public int accno;
    public String customer;
    public double balance;

    /*========================= ejbCreate methods ============================*/

    /**
     * There must be one ejbCreate() method per create() method on the Home interface, 
     *  and with the same signature.
     *
     * @param accno account number
     * @param customer customer name
     * @param balance initial balance
     *
     * @exception RemoteException If the instance could not perform the function 
     *            requested by the container 
     */
    public void ejbCreate(int val_accno, String val_customer, double val_balance) 
	throws RemoteException {

	// Init object state
	accno = val_accno;
	customer = val_customer;
	balance = val_balance;
    }
    
    /**
     * Each ejbCreate method should have a matching ejbPostCreate method
     */
    public void ejbPostCreate(int val_accno, String val_customer, double val_balance) 
	throws RemoteException {
	// Nothing to be done for this simple example.
    }



    /*====================== javax.ejb.EntityBean implementation =================*/

    /** 
     * A container invokes this method when the instance is taken out of the pool 
     *  of available instances to become associated with a specific EJB object. 
     * This method transitions the instance to the ready state. 
     * 
     * This method executes in an unspecified transaction context. 
     *
     * @exception RemoteException If the instance could not perform the function 
     *         requested by the container because of an system-level error. 
     */
    public void ejbActivate() throws RemoteException {
	// Nothing to be done for this simple example.
    }

    /**
     * A container invokes this method to instruct the instance to synchronize 
     *  its state by loading it state from the underlying database. 
     * This method always executes in the proper transaction context. 
     * 
     * @exception RemoteException  If the instance could not perform the function 
     *          requested by the container because of a system-level error. 
     *
     */
    public void ejbLoad() throws RemoteException {
	// Nothing to be done for this simple example, in implicit persistance.
    }

    /**
     * A container invokes this method on an instance before the instance becomes 
     *  disassociated with a specific EJB object. After this method completes, the 
     *  container will place the instance into the pool of available instances. 
     * 
     * This method executes in an unspecified transaction context.
     * 
     * @exception RemoteException  If the instance could not perform the function 
     *         requested by the container because of a system-level error. 
     */
    public void ejbPassivate() throws RemoteException {
	// Nothing to be done for this simple example.
    }

    /**
     * A container invokes this method before it removes the EJB object 
     *  that is currently associated with the instance. This method is invoked 
     *  when a client invokes a remove operation on the enterprise Bean's home 
     *  interface or the EJB object's remote interface. This method transitions 
     *  the instance from the ready state to the pool of available instances. 
     *
     * This method is called in the transaction context of the remove operation. 
     *
     * @exception RemoteException  Thrown if the instance could not perform the function 
     *         requested by the container because of a system-level error. 
     * @exception RemoveException  The enterprise Bean does not allow destruction of the object. 
     *
     */
    public void ejbRemove() throws RemoteException, RemoveException {
	// Nothing to be done for this simple example, in implicit persistance.
    }

    /**
     * A container invokes this method to instruct the instance to synchronize 
     *  its state by storing it to the underlying database. 
     * 
     * This method always executes in the proper transaction context. 
     * 
     * @exception: RemoteException  Thrown if the instance could not perform the function 
     *         requested by the container because of a system-level error. 
     */
    public void ejbStore() throws RemoteException {
	// Nothing to be done for this simple example, in implicit persistance.
    }
  

    /**
     * Sets the associated entity context. The container invokes this method 
     *  on an instance after the instance has been created. 
     * 
     * This method is called in an unspecified transaction context. 
     * 
     * @param ctx - An EntityContext interface for the instance. The instance should 
     *              store the reference to the context in an instance variable. 
     * @exception RemoteException  Thrown if the instance could not perform the function
     *        requested by the container because of a system-level error.
     */
    public void setEntityContext(EntityContext ctx) throws RemoteException { 

	// Keep the entity context in object
	entityContext = ctx;
    }


    /**
     * Unsets the associated entity context. The container calls this method 
     *  before removing the instance. This is the last method that the container 
     *  invokes on the instance. The Java garbage collector will eventually invoke 
     *  the finalize() method on the instance. 
     *
     * This method is called in an unspecified transaction context. 
     * 
     * @exception RemoteException  Thrown if the instance could not perform the function 
     *         requested by the container because of a system-level error.  
     */
    public void unsetEntityContext() {
	entityContext = null;
    }


    /*========================= Account implementation ============================*/

    /**
     * Business method for returning the balance.
     *
     * @return balance
     *
     */
    public double getBalance() throws RemoteException {

	return balance;
    }


    /**
     * Business method for updating the balance.
     *
     * @param d balance to update
     * 
     */
    public void setBalance(double d) throws RemoteException {

	balance = balance + d;
    }

  
    /**
     * Business method for returning the customer.
     *
     * @return customer
     *
     */
    public String  getCustomer() throws RemoteException {

	return customer;
    }


    /**
     * Business method for changing the customer name.
     *
     * @param c customer to update
     * 
     */
    public void setCustomer(String c) throws RemoteException {

	customer = c;
    }


    /**
     * Business method to get the Account number
     */
    public int getNumber() throws RemoteException {
	return accno;
    }
} 
