/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.adm;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import javax.naming.NamingEnumeration;
import javax.naming.NameClassPair;

import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.container.Container;


/**
 * This class implements a remote interface used for administering
 *      the server.
 *
 */
public class Adm extends UnicastRemoteObject implements AdmInterface {

    static final int trace = Trace.DB_18;

    /**
     * @serial
     */
    Container container = null;	// only 1 container in JOnAS for now

    /**
     * Adm constructor
     */
    public Adm(Container c) throws RemoteException  {
	Trace.outln(trace, "Adm.Adm() ");

	// keep the container
	container = c;

	// Registers the JOnAS Server Name to JNDI
	String name = Env.getJonasName() + "_Adm";
	try {
	    Env.getInitialContext().rebind(name, this);
	    Trace.outln(trace, "rebind done " + name);
	} catch (Exception e) {
	    Trace.errln("Cannot register "+name+" : " + e);
	    System.exit(1);
	}

    }

    // ------------------------------------------------------------------
    //  AdmInterface implementation
    // ------------------------------------------------------------------

    /**
     * configure trace messages in EJB Server
     */
    public void configureTrace(String out, String err, String level) throws RemoteException {
	Trace.out(trace, "Adm.configureTrace(String out, String err, String level) ");
	Trace.configure(out, err, level);	
    }

    /**
     * Add beans
     */
    public void addBeans(Vector descNames) throws RemoteException {
	Trace.out(trace, "Adm.addBeans(Vector descNames) ");
	container.addBeans(descNames);
    }

    /**
     * Remove beans
     */
    public void removeBeans(Vector descNames) throws RemoteException {
	Trace.out(trace, "Adm.removeBeans(Vector descNames) ");
	container.removeBeans(descNames);
    }

    /**
     * List beans
     */
    public Vector listBeans() throws RemoteException {
	Trace.out(trace, "Adm.listBeans() ");
	return container.listBeans();
    }
 
    /**
     * List JNDI context
     */
    public Vector listContext() {
	Trace.out(trace, "Adm.listContext() ");
	NamingEnumeration ne   = null;
	String            name = null;
	Vector            ret  = new Vector();
	String            url  = Prop.svr.getValue("java.naming.provider.url");
	try {
	    Trace.outln(trace, "    listing context of url = " + url);
	    ne = Env.getInitialContext().list(url);
	    for (Enumeration e = ne; e.hasMoreElements() ;) {
		name = ((NameClassPair) e.nextElement()).getName();
		Trace.outln(trace, "    name found in context = "+ name);
		ret.addElement(name);
	    }
	} catch (Exception ex) { 
	    Trace.errln("Cannot list JNDI context : " + ex);
	    System.exit(1);
	}
	return ret;
    }

    /**
     * List Environment
     */
    public Properties listEnv() {
	Trace.out(trace, "Adm.listEnv() ");
	return Prop.svr.getFilesEnv();
    }

    /**
     * Stop the EJB Server
     */
    public void stopServer()   throws RemoteException {
	Trace.out(trace, "Adm.stopServer() ");
	container.unbindBeans();
	container.unbindDataSources();
	System.exit(0);
    } 
}
