/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container; 

import java.rmi.RemoteException;
import javax.ejb.deployment.SessionDescriptor;
import javax.ejb.RemoveException;
import javax.ejb.Handle;
import javax.ejb.EJBObject;
import org.objectweb.jonas.common.Trace;

/**
 *
 */
public class JSessionHome extends JBeanHome {

    private final static int trace = Trace.DB_16;

    /**
     * @serial
     */
    private boolean isStateFul = false;

    /**
     * @serial
     */
    protected int sessionTimeOut = 0;

    /**
     * constructor
     */
    public JSessionHome(SessionDescriptor desc, String beanName) throws RemoteException {

	super(desc, beanName);

	Trace.outln(trace, "JSessionHome constructor for "+beanName);
	sessionTimeOut = desc.getSessionTimeout();
	if (desc.getStateManagementType() == SessionDescriptor.STATEFUL_SESSION) {
	    isStateFul = true;
	}

	JBeanMetaData bbmd = new JBeanMetaData(this, 
					       desc.getHomeInterfaceClassName(),
					       desc.getRemoteInterfaceClassName(),
					       null,
					       true,
					       !isStateFul);
	this.setEJBMetaData(bbmd);
    }
    
    public boolean isStateFul() {
	return isStateFul;
    }

    /**
     * Removes an EJB object identified by its handle. 
     */
    public void remove(Handle handle) throws RemoteException, RemoveException{
	Trace.outln(trace,"remove(Handle handle)");
	EJBObject ejbobj = handle.getEJBObject();
	ejbobj.remove();
    }

    /**
     * Removes an EJB object identified by its primary key.
     */

    public void remove(Object primaryKey) throws RemoteException, RemoveException {
	throw new RemoveException("There is no primary key on a sessionBean");
    }
}
