/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jta;

import java.io.Serializable;
import javax.transaction.xa.Xid;
import org.objectweb.jonas.common.Trace;

/**
 * Xid implementation for JTA
 */
public class XidImpl implements Xid, Serializable {

    private static final int trace = Trace.DB_5;

    private static final int JONAS_FORMAT_ID = 0xBB14;
    private static int count = 1;

    /**
     * @serial
     */
    private int formatId;
    /**
     * @serial
     */
    private byte[] gtrid;
    /**
     * @serial
     */
    private byte[] bqual;
    /**
     * @serial
     */
    private long uuid;
    /**
     * @serial
     */
    private int hashvalue;

    // -------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------

    /**
     * Build an Xid for a local transaction
     */
    public XidImpl() {

	Trace.outln(trace, "XidImpl constructor");

	formatId = JONAS_FORMAT_ID;
	synchronized(getClass()) {
	    hashvalue = count % 100;
	    uuid =  System.currentTimeMillis() * 1024 + count;
	    count++;
	}

	// compute gtrid
	String s = Long.toHexString(uuid);
	if (s.length() > Xid.MAXBQUALSIZE) {
	    s = s.substring(0, Xid.MAXBQUALSIZE);
	}
	gtrid = s.getBytes();

	// bqual is equal to gtrid in this constructor XXX
	bqual = s.getBytes();

	Trace.outln(trace, "new Xid = "+uuid);
    }

    // -------------------------------------------------------------------
    // Xid implementation
    // -------------------------------------------------------------------

    /**
     * Get the format id for that Xid
     */
    public int getFormatId() {
	return formatId;
    }

    /**
     * Get the Global Id for that Xid
     */
    public byte[] getGlobalTransactionId() {
	return gtrid;
    }

    /**
     * Get the Branch Qualifier for that Xid
     */
    public byte[] getBranchQualifier() {
	return bqual;
    }

    // -------------------------------------------------------------------
    // other methods
    // -------------------------------------------------------------------

    /**
     * get uuid
     */
    public long getUuid() {
	return uuid;
    }

    // -------------------------------------------------------------------
    // equals and hashCode
    // -------------------------------------------------------------------

    /**
     * return true if objects are identical
     */
    public boolean equals(Object obj2) {
	XidImpl xid2 = (XidImpl) obj2;

	if (formatId == xid2.getFormatId() && uuid == xid2.getUuid()) {
	    return true;
	} else {
	    return false;
	}
    }

    /**
     * return a hashcode value for this object
     */
    public int hashCode() {
	return hashvalue;	// Change this ! XXX
    }

}
