/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jtm;

import java.io.Serializable;
import java.util.Vector;
import javax.transaction.xa.Xid;
import org.objectweb.jonas.common.Trace;


/**
 * Otid associated to a transaction
 *
 * @see ControlImpl
 */
public class Otid implements Xid, Serializable {

    /**
     * Constructor for a JOnAS Otid, normal case.
     */
    public Otid(String server, int addr) {
	
	Trace.outln(trace, "Otid constructor: "+server+":"+addr);

	formatId = JONAS_FORMAT_ID;
	serverName = server;
	ipAddr = addr;
	synchronized(getClass()) {
	    uuid = System.currentTimeMillis() * 1024 + count++;
	}

	// compute JOnAS gtrid
	String s = Long.toHexString(uuid) + "_" + Long.toHexString(ipAddr) + "_" + serverName;
	if (s.length() > Xid.MAXBQUALSIZE) {
	    s = s.substring(0, Xid.MAXBQUALSIZE);
	}
	gtrid = s.getBytes();

	// bqual is equal to gtrid in this constructor (not a sub-coordinator)
	bqual = s.getBytes();
    }

    /**
     * Constructor in case of recreate. Only the bqual is build.
     */
    public Otid(Xid sup, String server, int addr) {

	Trace.outln(trace, "Otid constructor for subcoordinator: "+server+":"+addr);

	// Keep same FormatID
	formatId = sup.getFormatId();

	// Keep same gtrid
	gtrid = sup.getGlobalTransactionId();

	// Build a new branch qualifier
	serverName = server;
	ipAddr = addr;
	synchronized(getClass()) {
	    uuid = System.currentTimeMillis() * 1024 + count++;
	}
	String s = Long.toHexString(uuid) + "_" + Long.toHexString(ipAddr) + "_" + serverName;
	if (s.length() > Xid.MAXBQUALSIZE) {
	    s = s.substring(0, Xid.MAXBQUALSIZE);
	}
	bqual = s.getBytes();
    }

    /**
     * Get the format id for that Xid
     */
    public int getFormatId() {
	return formatId;
    }

    /**
     * Get the Global Id for that Xid
     */
    public byte[] getGlobalTransactionId() {
	return gtrid;
    }

    /**
     * Get the Branch Qualifier for that Xid
     */
    public byte[] getBranchQualifier() {
	return bqual;
    }

    private static int trace = Trace.DB_4;
    private static int count = 0;

    /**
     * @serial
     */
    private int formatId;
    private final static int JONAS_FORMAT_ID = 0xBB20;

    /**
     * @serial
     */
    private byte[] gtrid;

    /**
     * @serial
     */
    private byte[] bqual;

    /**
     * @serial
     */
    private long uuid;	// bqual only

    /**
     * @serial
     */	
    private String serverName;	// bqual only

    /**
     * @serial
     */
    private int ipAddr;		// bqual only
}
