/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jtm;

import java.io.*;
import javax.naming.Context;      // jndi
import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.Trace;


/**
 * Clock for TMServer
 *
 * Every second, decrement timers and launch action if expired
 */
class Clock extends Thread {

    public Clock() {
	Trace.outln(TMServer.trace, "Clock constructor");
    }
    
    public void run() {
	Timer.updateAll();
    }
}

/**
 * Batch for TMServer
 *
 */
class Batch extends Thread {

    public Batch() {
	Trace.outln(TMServer.trace, "Batch constructor");
    }
    
    public void run() {
	Timer.processAll();
    }
}

/**
 * TMServer
 */
public class TMServer {

    /**
     * Gets the local transaction factory
     */
    public static TransactionFactoryImpl getTransactionFactory() {
	return tm;
    }

    /**
     * initilizes the TM. Can be from remote TM, or from EJBServer.
     */
    public static TransactionFactory init() {

	// Start recovery ???
	// XXX

	// Create and register TM factory
	Context initialContext = Env.getInitialContext();
	if (initialContext == null) {
	    Trace.errln("Cannot get initial context for JNDI");
	    System.exit(2);
	}
	try {
	    Trace.outln(trace, "Create and register TM factory ");
	    tm = new TransactionFactoryImpl();
	    initialContext.rebind("TMFactory", tm);
	} catch (Exception e) {
	    Trace.errln("Cannot rebind TM:\n"+e);
	    System.exit(2);
	}

	// return the Transaction Factory
	return tm;
    }

    public static void main(String[] args) {
	
	Trace.configure();
	init();

	// Create threads for timers
	new Batch().start();
	new Clock().start();

	Trace.outln(trace, "TM is ready");
    }

    static TransactionFactoryImpl tm = null;

    static int trace = Trace.DB_4;
}
