/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.lib;

import java.io.Serializable;
import java.lang.Object;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;

/**
 * This class implements the java.util.Enumeration and the java.io.Serializable interfaces.
 * <br>
 * This class is used as the return value type of the implementation of the finder methods
 * which return a collection.
 * <br>
 * Indeed, the EJB spec. tells that the type for a collection is the java.util.Enumeration.
 * In addition the return value type must be legal type for Java RMI (ie. serializable type).
 */

public class CollectionEnum implements Serializable, Enumeration {

    /**
     * @serial
     */
    private Vector mCollection = null;
    /**
     * @serial
     */
    private int mIndex = 0;

    /**
     * Create an empty CollectionEnum
     *
     */
    public CollectionEnum() {
	mCollection = new Vector();
	mIndex = 0;
    }

    /**
     * Add an element
     */
    public synchronized void addElement(Object obj) {
	mCollection.addElement(obj);
    }

    /**
     * Create a CollectionEnum from a vector
     */
    public CollectionEnum(Vector v) {
	mCollection = new Vector();
	for (int i=0; i<v.size(); i++) {
	    mCollection.addElement(v.elementAt(i));
	}
	mIndex = 0;
    }

    /**
     * Implements Enumeration.hasMoreElements()
     */
    public boolean hasMoreElements() {
	return(mIndex<mCollection.size());
    }

    /**
     * Implements Enumeration.nextElement()
     */
    public Object nextElement() throws NoSuchElementException {
	if (mIndex>=mCollection.size()) {
	    throw new NoSuchElementException("CollectionEnum ("+mIndex+">="+mCollection.size()+")");
	}
	mIndex++;
	return(mCollection.elementAt(mIndex-1));
    }

}

