/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.BeanNaming;
import org.objectweb.jonas.lib.DD;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Entity Enterprise Java Bean
 *  with container managed persistence.
 *  Its extends GenICEntityHome class, itself extends GenICHome.
 */
abstract class GenICEntityCMHome extends GenICEntityHome {

    Field[] fieldsPK = null;

    /**
     * GenICEntityCMHome Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     *                 with container managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityCMHome(DD dd, String fileName) throws GenICException {
	super(dd, fileName);
	// Get the list of the PrimaryKey fields
	// and check if all fields can be treated
	try {
	    fieldsPK = ddesc.getFieldsOfPrimaryKey();
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
	for (int i = 0; i<fieldsPK.length; i++) {
	    if ((JavaType.getSQLGetMethod(fieldsPK[i].getType()) == null) &&
		(JavaType.getSQLSetMethod(fieldsPK[i].getType()) == null)) {
		throw new GenICException("Cannot container manage the primary key's field '" +
					 fieldsPK[i].getName() +
					 "' (Type '" + JavaType.getName(fieldsPK[i].getType()) + "')");
	    }
	}
    }

    /**
     * Source generation for the given create() method described in the Home interface
     * @exception GenICException in error case
     */
    void genCreate(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();

	genHeaderMethod(method);
	src.println(wrpRemoteBaseName + " ejbobj;");
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println(ejbBeanName + " eb = null;");
	src.println("EntityContextImpl ctx = null;");
	src.println("try {");
	src.indentPlus();
	src.println("ctx = ejbobj.createContext();");
	src.println("eb = ("+ejbBeanName +")ctx.getInstance();");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to create a new instance of the bean class "+ejbBeanName+"\");");
	src.println("Trace.errln(e);");
	src.println("throw new RemoteException(\"Failed to create a new instance of the bean class "+ejbBeanName+"\", e);");
	src.indentMinus();
	src.println("}");
	src.println(ejbPrimaryKeyName + " pk;");
	src.println("try {");
	src.indentPlus();
	src.print  ("eb.ejbCreate(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBeanException(method, "ejbCreate");
	src.indentMinus();
	src.println("}");
	src.println("pk = new " + ejbPrimaryKeyName + "();");
	for (int f=0; f<fieldsPK.length; f++) {
	    src.println("pk." + fieldsPK[f].getName() +
			" = eb." + fieldsPK[f].getName() + ";");
	}
	src.println("ctx.setPrimaryKey(pk);");
	src.println("ejbobj.createData();");
	src.println("try {");
	src.indentPlus();
	src.print  ("eb.ejbPostCreate(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBeanException(method, "ejbPostCreate");
	src.indentMinus();
	src.println("}");
	src.println("putEJBObject(pk, ejbobj);");
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindByPK(Method method) throws GenICException {

	super.genFindByPK(method);

	Class params[] = method.getParameterTypes();
	Class exceptions[] = method.getExceptionTypes();

	/*
	 * findByPrimaryKey(PrimaryKey p1, boolean callEjbFindByPK) Method
	 */
	src.print  ("synchronized public " +
		    JavaType.getName(method.getReturnType()) +
		    " findByPrimaryKey(");
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]) + " " + "p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.print(", boolean callEjbFindByPK) ", false);
	for (int e=1; e <= exceptions.length; e++) {
	    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptions[e-1].getName(), false);
	    if (e < exceptions.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);
	// begining of the core
	src.indentPlus();
	src.println(wrpRemoteBaseName + " ejbobj = (" +
		    wrpRemoteBaseName + ")getEJBObject(p1);");
	src.println("if (ejbobj != null) {");
	src.indentPlus();
	src.println("return((" + ejbRemoteName+ ") ejbobj);");
	src.indentMinus();
	src.println("}");
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println("EntityContextImpl ctx = null;");
	src.println("try {");
	src.indentPlus();
	src.println("ctx = ejbobj.createContext();");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to create a new instance of the bean class "+ejbBeanName+"\");");
	src.println("Trace.errln(e);");
	src.println("throw new RemoteException(\"Failed to create a new instance of the bean class "+ejbBeanName+"\", e);");
	src.indentMinus();
	src.println("}");
	src.println(ejbPrimaryKeyName + " pk;");
	src.println("if (callEjbFindByPK) {");
	src.indentPlus();
	src.println("pk = pkFindByPrimaryKey(p1);");
	src.indentMinus();
	src.println("} else {");
	src.indentPlus();
	src.println("pk = p1;");
	src.indentMinus();
	src.println("}");
	src.println("ctx.setPrimaryKey(pk);");
	src.println("putEJBObject(pk, ejbobj);");
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");

	src.println();
	genEjbFindByPK(method);
    }

    /**
     * Source generation of the ejbFindByPrimaryKey() method associated to the given
     * findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void genEjbFindByPK(Method method) throws GenICException;

    /**
     * Source generation for the given findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindOther(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();
	boolean isWithCollection = ! method.getReturnType().equals(ejbRemoteClass);

	genHeaderMethod(method);
	if (!isWithCollection) {
	    src.println(ejbRemoteName + " ejbobj = null;");
	    src.println(ejbPrimaryKeyName + " pk;");
	    src.print  ("pk = pk" +
			BeanNaming.firstToUpperCase(method.getName()) +
			"(");
	    for (int p=1; p <= params.length; p++) {
		src.print("p" + p, false);
		if (p < params.length) {
		    src.print(", ", false);
		}
	    }
	    src.println(");", false);
	    src.println("ejbobj = findByPrimaryKey(pk, false);");
	    src.println("return(ejbobj);");
	} else {
	    // findXXX() method with a Collection
	    boolean isEnumerationReturnType = method.getReturnType().equals(java.util.Enumeration.class);
	    if (isEnumerationReturnType) {
		src.println("CollectionEnum ejbobjV = new CollectionEnum();");
	    } else {
		//java.util.Collection
		src.println("java.util.Vector ejbobjV = new java.util.Vector();");
	    }
	    src.println(ejbRemoteName + " ejbobj = null;");
	    src.println("Enumeration pkC = null;");
	    src.println(ejbPrimaryKeyName + " pk;");
	    src.print  ("pkC = pk" +
			BeanNaming.firstToUpperCase(method.getName()) +
			"(");
	    for (int p=1; p <= params.length; p++) {
		src.print("p" + p, false);
		if (p < params.length) {
		    src.print(", ", false);
		}
	    }
	    src.println(");", false);
	    src.println("while (pkC.hasMoreElements()) {");
	    src.indentPlus();
	    src.println("ejbobj = findByPrimaryKey(("+
			ejbPrimaryKeyName+")pkC.nextElement(), false);");
	    src.println("ejbobjV.addElement((Object)ejbobj);");
	    src.indentMinus();
	    src.println("}");
	    src.println("return(ejbobjV);");
	}
	src.indentMinus();
	src.println("}");

	src.println();
	genEjbFindOther(method);
    }

    /**
     * Source generation of the ejbFindXXX() method associated to the given
     * findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void genEjbFindOther(Method method) throws GenICException;

}


