/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.DD;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Entity Enterprise Java Bean.
 *  Its extends GenICHome class.
 */
abstract class GenICEntityHome extends GenICHome {

    // Full names of the interfaces and the classes
    String ejbPrimaryKeyName = null;

    /**
     * GenICEntityHome Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityHome (DD dd, String fileName) throws GenICException {
	super(dd, fileName);

	ejbPrimaryKeyName = (ddesc.getEntityDescriptor()).getPrimaryKeyClassName();
	if ((ejbPrimaryKeyName == null) || (ejbPrimaryKeyName.length() < 1)) {
	    throw new GenICException("The PrimaryKeyClassName MUST BE defined "+
				     "in the EntityDescriptor");
	}

    }

    /**
     * Name of the superclass of the generated class
     */
    String getSuperClass() {
	return(new String("JEntityHome"));
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {

	src.println("public " + wrpHomeBaseName +
		    "(EntityDescriptor desc, String BeanName, Object ds) throws RemoteException {");
	src.indentPlus();
	src.println("super(desc, BeanName, ds);");
	src.println("Trace.outln(Trace.DB_15, \"" + wrpHomeBaseName + "(\"+BeanName+\")\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindByPK(Method method) throws GenICException {

	Class exceptions[] = method.getExceptionTypes();

	/*
	 * XXX findByPrimaryKey(PrimaryKey p1) Method
	 */
	src.print("public " + JavaType.getName(method.getReturnType()) +
		  " findByPrimaryKey("+ejbPrimaryKeyName+" p1) ");
	for (int e=1; e <= exceptions.length; e++) {
		    if (e==1) {
		src.print("throws ", false);
	    }
	    src.print(exceptions[e-1].getName(), false);
	    if (e < exceptions.length) {
		src.print(",", false);
	    }
	    src.print(" ", false);
	}
	src.println("{", false);
	// begining of the core
	src.indentPlus();
	src.println("return(findByPrimaryKey(p1, true));");
	src.indentMinus();
	src.println("}");
	src.println();
    }


    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

    /**
     * Source generation for the given create() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genCreate(Method method) throws GenICException;

    /**
     * Source generation for the given findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genFindOther(Method method) throws GenICException;

}

